@@ -1,7 +1,6 @@
 import argparse
 import logging
 import os
-import yaml
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
@@ -10,6 +9,7 @@ from juju.agents.base import TwistedOptionNamespace
 from juju.charm import get_charm_from_path
 from juju.charm.url import CharmURL
 from juju.errors import JujuError
+from juju.lib import serializer
 from juju.state.environment import GlobalSettingsStateManager
 from juju.state.errors import ServiceStateNotFound
 from juju.state.service import NO_HOOKS, RETRY_HOOKS
@@ -375,7 +375,7 @@ class UnitAgentTest(UnitAgentTestBase):
         self.assertEqual(
             {"greeting": "hello", "planet": "earth",
              "private-address": "mysql-0.example.com"},
-            yaml.load(contents))
+            serializer.load(contents))
 
         self.failUnlessIn("wordpress/0", output)
 
@@ -643,7 +643,6 @@ class UnitAgentUpgradeTest(
         self.assertEquals(
             (yield self.states["unit"].get_charm_id()), old_charm_id)
 
-
     @inlineCallbacks
     def test_agent_upgrade_bad_unit_state(self):
         """The upgrade fails if the unit is in a bad state."""
@@ -672,6 +671,36 @@ class UnitAgentUpgradeTest(
         self.assertEquals(
             (yield self.states["unit"].get_charm_id()), old_charm_id)
 
+    @inlineCallbacks
+    def test_agent_force_upgrade_bad_unit_state(self):
+        """The upgrade runs if forced and the unit is in a bad state."""
+        # Upload a new version of the unit's charm
+        repository = self.increment_charm(self.charm)
+        charm = yield repository.find(CharmURL.parse("local:series/mysql"))
+        charm, charm_state = yield self.publish_charm(charm.path)
+        old_charm_id = yield self.states["unit"].get_charm_id()
+
+        output = self.capture_logging("juju.agents.unit", level=logging.DEBUG)
+
+        self.agent.set_watch_enabled(True)
+        yield self.agent.startService()
+
+        # Mark the unit for upgrade, with an invalid state.
+        with (yield self.agent.workflow.lock()):
+            yield self.agent.workflow.fire_transition("error_configure")
+        yield self.states["service"].set_charm_id(charm_state.id)
+        yield self.states["unit"].set_upgrade_flag(force=True)
+
+        # Its hard to watch something with no hooks and no state changes.
+        yield self.sleep(0.1)
+
+        self.assertIdentical(
+            (yield self.states["unit"].get_upgrade_flag()), False)
+        self.assertIn("Forced upgrade complete", output.getvalue())
+        self.assertEquals(
+            (yield self.states["unit"].get_charm_id()), "local:series/mysql-2")
+        self.assertEquals(old_charm_id, "local:series/dummy-1")
+
     @inlineCallbacks
     def test_agent_upgrade_no_flag(self):
         """An upgrade stops if there is no upgrade flag set."""
