@@ -2,21 +2,23 @@ import json
 import os
 import inspect
 import shutil
-import tempfile
 
 from twisted.internet.defer import fail, inlineCallbacks, succeed
 from twisted.web.error import Error
+from txaws.client.ssl import VerifyingContextFactory
+
 
 from juju.charm.directory import CharmDirectory
 from juju.charm.errors import CharmNotFound, CharmURLError, RepositoryNotFound
 from juju.charm.repository import (
-    LocalCharmRepository, RemoteCharmRepository, resolve)
+    LocalCharmRepository, RemoteCharmRepository, resolve, CS_STORE_URL)
 from juju.charm.url import CharmURL
+from juju.charm import provider
 from juju.errors import CharmError
 from juju.lib import under
 
 from juju.charm import tests
-from juju.lib.mocker import ANY
+from juju.lib.mocker import ANY, MATCH
 from juju.lib.testing import TestCase
 
 
@@ -57,6 +59,25 @@ class LocalRepositoryTest(RepositoryTestBase):
         # define repository objects
         self.repository1 = LocalCharmRepository(self.unbundled_repo_path)
         self.repository2 = LocalCharmRepository(self.bundled_repo_path)
+        self.output = self.capture_logging("juju.charm")
+
+    def assert_there(self, name, repo, revision, latest_revision=None):
+        url = self.charm_url(name)
+        charm = yield repo.find(url)
+        self.assertEquals(charm.get_revision(), revision)
+        latest = yield repo.latest(url)
+        self.assertEquals(latest, latest_revision or revision)
+
+    @inlineCallbacks
+    def assert_not_there(self, name, repo, revision=None):
+        url = self.charm_url(name)
+        msg = "Charm 'local:series/%s' not found in repository %s" % (
+            name, repo.path)
+        err = yield self.assertFailure(repo.find(url), CharmNotFound)
+        self.assertEquals(str(err), msg)
+        if revision is None:
+            err = yield self.assertFailure(repo.latest(url), CharmNotFound)
+            self.assertEquals(str(err), msg)
 
     def charm_url(self, name):
         return CharmURL.parse("local:series/" + name)
@@ -74,17 +95,6 @@ class LocalRepositoryTest(RepositoryTestBase):
         err = self.assertRaises(RepositoryNotFound, LocalCharmRepository, path)
         self.assertEquals(str(err), "No repository found at %r" % path)
 
-    @inlineCallbacks
-    def assert_not_there(self, name, repo, revision=None):
-        url = self.charm_url(name)
-        msg = "Charm 'local:series/%s' not found in repository %s" % (
-            name, repo.path)
-        err = yield self.assertFailure(repo.find(url), CharmNotFound)
-        self.assertEquals(str(err), msg)
-        if revision is None:
-            err = yield self.assertFailure(repo.latest(url), CharmNotFound)
-            self.assertEquals(str(err), msg)
-
     def test_find_inappropriate_url(self):
         url = CharmURL.parse("cs:foo/bar")
         err = self.assertRaises(AssertionError, self.repository1.find, url)
@@ -93,12 +103,31 @@ class LocalRepositoryTest(RepositoryTestBase):
     def test_completely_missing(self):
         return self.assert_not_there("zebra", self.repository1)
 
-    def test_unkown_files_ignored(self):
+    def test_unknown_files_ignored(self):
         self.makeFile(
             "Foobar",
             path=os.path.join(self.repository1.path, "series", "zebra"))
         return self.assert_not_there("zebra", self.repository1)
 
+    @inlineCallbacks
+    def test_random_error_logged(self):
+        get_charm = self.mocker.replace(provider.get_charm_from_path)
+        get_charm(ANY)
+        self.mocker.throw(SyntaxError("magic"))
+        self.mocker.count(0, 3)
+        self.mocker.replay()
+
+        yield self.assertFailure(
+            self.repository1.find(self.charm_url("zebra")),
+            CharmNotFound)
+
+        self.assertIn(
+            "Unexpected error while processing",
+            self.output.getvalue())
+        self.assertIn(
+            "SyntaxError('magic',)",
+            self.output.getvalue())
+
     def test_unknown_directories_ignored(self):
         self.makeDir(
             path=os.path.join(self.repository1.path, "series", "zebra"))
@@ -116,10 +145,9 @@ requires: {interface: zebra
 revision: 0
 summary: hola""")
         fh.close()
-        stream = self.capture_logging("juju.charm")
         yield self.assertFailure(
             self.repository1.find(self.charm_url("zebra")), CharmNotFound)
-        output = stream.getvalue()
+        output = self.output.getvalue()
         self.assertIn(
             "Charm 'zebra' has a YAML error", output)
         self.assertIn(
@@ -140,14 +168,27 @@ requires: {interface: zebra
 revision: 0
 summary: hola""")
         fh.close()
-        stream = self.capture_logging("juju.charm")
         yield self.repository1.find(self.charm_url("sample"))
-        output = stream.getvalue()
+        output = self.output.getvalue()
         self.assertIn(
             "Charm 'mysql' has a YAML error", output)
         self.assertIn(
             "%s/series/mysql/config.yaml" % self.repository1.path, output)
 
+    @inlineCallbacks
+    def test_ignore_dot_files(self):
+        """Dot files are ignored when browsing the repository."""
+        fh = open(
+            os.path.join(
+                self.repository1.path, "series", ".foo"),
+            "w+")
+
+        fh.write("Something")
+        fh.close()
+        yield self.repository1.find(self.charm_url("sample"))
+        output = self.output.getvalue()
+        self.assertNotIn("Charm '.foo' has an error", output)
+
     @inlineCallbacks
     def test_invalid_charm_config_ignored(self):
         fh = open(
@@ -169,13 +210,6 @@ summary: hola""")
     def test_repo_type(self):
         self.assertEqual(self.repository1.type, "local")
 
-    def assert_there(self, name, repo, revision, latest_revision=None):
-        url = self.charm_url(name)
-        charm = yield repo.find(url)
-        self.assertEquals(charm.get_revision(), revision)
-        latest = yield repo.latest(url)
-        self.assertEquals(latest, latest_revision or revision)
-
     @inlineCallbacks
     def test_success_unbundled(self):
         yield self.assert_there("sample", self.repository1, 2)
@@ -248,15 +282,19 @@ class RemoteRepositoryTest(RepositoryTestBase):
         return json.dumps({url_str: info})
 
     def mock_charm_info(self, url, result):
-        self.getPage(url)
+        def match_context(value):
+            return isinstance(value, VerifyingContextFactory)
+        self.getPage(url, contextFactory=MATCH(match_context))
         self.mocker.result(result)
 
     def mock_download(self, url, error=None):
-        self.downloadPage(url, ANY)
+        def match_context(value):
+            return isinstance(value, VerifyingContextFactory)
+        self.downloadPage(url, ANY, contextFactory=MATCH(match_context))
         if error:
             return self.mocker.result(fail(error))
 
-        def download(_, path):
+        def download(_, path, contextFactory):
             self.assertTrue(path.startswith(self.download_path))
             with open(path, "wb") as f:
                 f.write(self.bundle_data)
@@ -516,7 +554,7 @@ class ResolveTest(RepositoryTestBase):
         repo, url = resolve(vague, None, default)
         self.assertEquals(str(url), expect)
         self.assertTrue(isinstance(repo, RemoteCharmRepository))
-        self.assertEquals(repo.url_base, "https://store.juju.ubuntu.com")
+        self.assertEquals(repo.url_base, CS_STORE_URL)
 
     def test_resolve_remote(self):
         self.assert_resolve_remote(
