@@ -1,9 +1,8 @@
-from yaml import dump
-
 from twisted.internet.defer import inlineCallbacks
 
 from juju.control import main
 from juju.control.config_set import config_set
+from juju.lib import serializer
 from .common import MachineControlToolTest
 
 
@@ -16,6 +15,7 @@ class ControlJujuSetTest(MachineControlToolTest):
         self.service_unit = yield self.service_state.add_unit_state()
         self.environment = self.config.get_default()
         self.stderr = self.capture_stream("stderr")
+        self.output = self.capture_logging()
 
     @inlineCallbacks
     def test_set_and_get(self):
@@ -37,8 +37,8 @@ class ControlJujuSetTest(MachineControlToolTest):
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
         self.mocker.replay()
-        config_file = self.makeFile(dump(dict(
-            wordpress={"blog-title": "Hello World"})))
+        config_file = self.makeFile(serializer.yaml_dump(
+            dict(wordpress={"blog-title": "Hello World"})))
 
         main(["set", "wordpress",
               "--config=%s" % config_file])
@@ -53,13 +53,13 @@ class ControlJujuSetTest(MachineControlToolTest):
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
         self.mocker.replay()
+
         # missing the service_name dict (will do nothing to values)
-        config_file = self.makeFile(dump({"blog-title": "Hello World"}))
-
-        main(["set", "wordpress",
-              "--config=%s" % config_file])
-
+        config_file = self.makeFile(
+            serializer.yaml_dump({"blog-title": "Hello World"}))
+        main(["set", "wordpress", "--config=%s" % config_file])
         yield finished
+
         state = yield self.service_state.get_config()
         self.assertEqual(state, {'blog-title': 'My Title'})
 
@@ -89,7 +89,7 @@ class ControlJujuSetTest(MachineControlToolTest):
         self.mocker.replay()
 
         # valid file, but incorrect cli usage
-        config_file = self.makeFile(dump(dict(
+        config_file = self.makeFile(serializer.yaml_dump(dict(
             wordpress={"blog-title": "Hello World"})))
 
         main(["-v", "set", "wordpress",
@@ -112,7 +112,7 @@ class ControlJujuSetTest(MachineControlToolTest):
 
         # Make sure we got an error message to the user
         self.assertIn("blog-roll is not a valid configuration option.",
-                      self.stderr.getvalue())
+                      self.output.getvalue())
 
     @inlineCallbacks
     def test_set_invalid_service(self):
@@ -126,7 +126,7 @@ class ControlJujuSetTest(MachineControlToolTest):
         yield finished
 
         self.assertIn("Service 'whatever' was not found",
-                      self.stderr.getvalue())
+                      self.output.getvalue())
 
     @inlineCallbacks
     def test_set_valid_option(self):
@@ -184,55 +184,49 @@ class ControlJujuSetTest(MachineControlToolTest):
                                  "bar": "new bar"})
 
     @inlineCallbacks
-    def test_boolean_option_invalid_format_v1(self):
-        """Verify not possible to set a boolean option with format v1"""
+    def test_boolean_option_str_format_v1(self):
+        """Verify possible to set a boolean option with format v1"""
         self.service_state = yield self.add_service_from_charm("mysql")
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
         self.mocker.replay()
-        main(["set", "mysql","awesome=true"])
+        main(["set", "mysql", "awesome=true"])
         yield finished
-        self.assertEqual(
-            self.stderr.getvalue(),
-            "Invalid value for awesome: 'true'\n")
         state = yield self.service_state.get_config()
         self.assertEqual(
             state,
-            {"awesome": False, "monkey-madness": 0.5,
+            {"awesome": True, "monkey-madness": 0.5,
              "query-cache-size": -1, "tuning-level": "safest"})
 
     @inlineCallbacks
     def test_int_option_coerced_format_v1(self):
-        """Verify int feasible, but coerced internally to str in format v1"""
+        """Verify int option possible in format v1"""
         self.service_state = yield self.add_service_from_charm("mysql")
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
         self.mocker.replay()
-        main(["set", "mysql","query-cache-size=10"])
+        main(["set", "mysql", "query-cache-size=10"])
         yield finished
         self.assertEqual(self.stderr.getvalue(), "")
         state = yield self.service_state.get_config()
         self.assertEqual(
             state,
             {"awesome": False, "monkey-madness": 0.5,
-             "query-cache-size": "10", "tuning-level": "safest"})
+             "query-cache-size": 10, "tuning-level": "safest"})
 
     @inlineCallbacks
-    def test_float_option_invalid_format_v1(self):
-        """Verify not possible to set a float option with format v1"""
+    def test_float_option_str_format_v1(self):
+        """Verify possible to set a float option with format v1"""
         self.service_state = yield self.add_service_from_charm("mysql")
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
         self.mocker.replay()
-        main(["set", "mysql","monkey-madness=0.99999999"])
+        main(["set", "mysql", "monkey-madness=0.99999999"])
         yield finished
-        self.assertEqual(
-            self.stderr.getvalue(),
-            "Invalid value for monkey-madness: '0.99999999'\n")
         state = yield self.service_state.get_config()
         self.assertEqual(
             state,
-            {"awesome": False, "monkey-madness": 0.5,
+            {"awesome": False, "monkey-madness": 0.99999999,
              "query-cache-size": -1, "tuning-level": "safest"})
 
     @inlineCallbacks
@@ -266,10 +260,10 @@ class ControlJujuSetTest(MachineControlToolTest):
         self.setup_exit(0)
         self.mocker.replay()
         main(["set", "mysql-format-v2",
-              "monkey-madness='barrels of monkeys'"])
+              "monkey-madness=barrels of monkeys"])
         yield finished
         self.assertEqual(
-            self.stderr.getvalue(), 
+            self.output.getvalue(),
             "Invalid value for monkey-madness: barrels of monkeys\n")
         state = yield self.service_state.get_config()
         self.assertEqual(
@@ -288,31 +282,10 @@ class ControlJujuSetTest(MachineControlToolTest):
         main(["set", "mysql-format-v2", "query-cache-size=big"])
         yield finished
         self.assertEqual(
-            self.stderr.getvalue(), 
+            self.output.getvalue(),
             "Invalid value for query-cache-size: big\n")
         state = yield self.service_state.get_config()
         self.assertEqual(
             state,
             {"awesome": False, "monkey-madness": 0.5,
              "query-cache-size": -1, "tuning-level": "safest"})
-
-    @inlineCallbacks
-    def test_invalid_string_option_format_v2(self):
-        """Verify that config settings reject invalid string"""
-        self.service_state = yield self.add_service_from_charm(
-            "mysql-format-v2")
-        finished = self.setup_cli_reactor()
-        self.setup_exit(0)
-        self.mocker.replay()
-        # YAML does a fair amount of coercion... note that it needs to
-        # be quoted if it looks like a boolean/int/string
-        main(["set", "mysql-format-v2", "tuning-level=FALSE"])
-        yield finished
-        self.assertEqual(
-            self.stderr.getvalue(), 
-            "Invalid value for tuning-level: false\n")
-        state = yield self.service_state.get_config()
-        self.assertEqual(
-            state,
-            {"awesome": False, "monkey-madness": 0.5,
-             "query-cache-size": -1, "tuning-level": "safest"})
