@@ -6,7 +6,7 @@ from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
 from twisted.python.failure import Failure
 
 from juju import errors
-from juju.state.errors import RelationStateNotFound
+from juju.state.errors import RelationStateNotFound, InvalidRelationIdentity
 from juju.state.hook import RelationHookContext
 
 
@@ -38,7 +38,7 @@ class HookProtocol(protocol.ProcessProtocol):
 
     def errReceived(self, data):
         """Log `data` from stderr until the child process has ended."""
-        self._log.error(data)
+        self._log.info(data)
 
     def _process_reason(self, reason, deferred):
         """Common code for working with both processEnded and processExited.
@@ -239,7 +239,11 @@ class Invoker(object):
         try:
             return self._relation_contexts[relation_ident]
         except KeyError:
-            raise RelationStateNotFound()
+            parts = relation_ident.split(":")
+            if len(parts) != 2 or not parts[1].isdigit():
+                raise InvalidRelationIdentity(relation_ident)
+            else:
+                raise RelationStateNotFound()
 
     def get_relation_idents(self, relation_name):
         return self._context.get_relation_idents(relation_name)
