@@ -48,11 +48,13 @@ import logging
 from twisted.internet import defer
 from twisted.internet import protocol
 from twisted.protocols import amp
-import yaml
+
 
 from juju.errors import JujuError
+from juju.lib import serializer
 from juju.lib.format import get_charm_formatter, get_charm_formatter_from_env
-from juju.state.errors import UnitRelationStateNotFound
+from juju.state.errors import (
+    InvalidRelationIdentity,  RelationStateNotFound, UnitRelationStateNotFound)
 from juju.state.hook import RelationHookContext
 
 
@@ -83,11 +85,15 @@ class MustSpecifyRelationName(JujuError):
 
 
 class BaseCommand(amp.Command):
-    errors = {NoSuchUnit: "NoSuchUnit",
-              NoSuchKey: "NoSuchKey",
-              NotRelationContext: "NotRelationContext",
-              UnitRelationStateNotFound: "UnitRelationStateNotFound",
-              MustSpecifyRelationName: "MustSpecifyRelationName"}
+    errors = {
+        NoSuchUnit: "NoSuchUnit",
+        NoSuchKey: "NoSuchKey",
+        NotRelationContext: "NotRelationContext",
+        UnitRelationStateNotFound: "UnitRelationStateNotFound",
+        MustSpecifyRelationName: "MustSpecifyRelationName",
+        InvalidRelationIdentity: "InvalidRelationIdentity",
+        RelationStateNotFound: "RelationStateNotFound"
+        }
 
 
 # All the commands below this point should be documented in the
@@ -310,7 +316,7 @@ class UnitAgentServer(amp.AMP, object):
 
         # NOTE: no need to consider charm format for blob here, this
         # blob has always been in YAML format
-        defer.returnValue(dict(data=yaml.safe_dump(options)))
+        defer.returnValue(dict(data=serializer.dump(options)))
 
     @OpenPortCommand.responder
     @defer.inlineCallbacks
@@ -328,6 +334,13 @@ class UnitAgentServer(amp.AMP, object):
         """
         context = self.factory.get_context(client_id)
         service_unit_state = yield context.get_local_unit_state()
+        container = yield service_unit_state.get_container()
+        if container:
+            # also open the port on the container
+            # we still do subordinate unit below to ease
+            # status reporting
+            yield container.open_port(port, proto)
+
         yield service_unit_state.open_port(port, proto)
         yield self.factory.log(logging.DEBUG, "opened %s/%s" % (port, proto))
         defer.returnValue({})
@@ -348,6 +361,13 @@ class UnitAgentServer(amp.AMP, object):
         """
         context = self.factory.get_context(client_id)
         service_unit_state = yield context.get_local_unit_state()
+        container = yield service_unit_state.get_container()
+        if container:
+            # also close the port on the container
+            # we still do subordinate unit below to ease
+            # status reporting
+            yield container.close_port(port, proto)
+
         yield service_unit_state.close_port(port, proto)
         yield self.factory.log(logging.DEBUG, "closed %s/%s" % (port, proto))
         defer.returnValue({})
@@ -443,7 +463,7 @@ class UnitAgentClient(amp.AMP, object):
                                        client_id=client_id,
                                        option_name=option_name)
         # Unbundle and deserialize
-        result = yaml.safe_load(result["data"])
+        result = serializer.load(result["data"])
         defer.returnValue(result)
 
     @defer.inlineCallbacks
