@@ -1,12 +1,12 @@
 # -*- encoding: utf-8 -*-
 
 from StringIO import StringIO
+import base64
 import json
 import logging
 import os
 import stat
 import sys
-import yaml
 
 from twisted.internet import defer
 from twisted.internet.process import Process
@@ -19,6 +19,7 @@ from juju.lib.pick import pick_attr
 from juju.hooks import invoker
 from juju.hooks import commands
 from juju.hooks.protocol import UnitSettingsFactory
+from juju.lib import serializer
 from juju.lib.mocker import MATCH
 from juju.lib.twistutils import get_module_directory
 from juju.state import hook
@@ -122,6 +123,21 @@ class MockUnitAgent(object):
             self.socket_path, self.server_factory)
 
 
+def capture_separate_log(name, level):
+    """Support the separate capture of logging at different log levels.
+
+    TestCase.capture_logging only allows one level to be captured at
+    any given time. Given that the hook log captures both data AND
+    traditional logging, it's useful to separate.
+    """
+    logger = logging.getLogger(name)
+    output = StringIO()
+    handler = logging.StreamHandler(output)
+    handler.setLevel(level)
+    logger.addHandler(handler)
+    return output
+
+
 def get_cli_environ_path(*search_path):
     """Construct a path environment variable.
 
@@ -192,11 +208,36 @@ class InvokerTestBase(EnvironmentsConfigTestBase):
 
     def create_hook(self, hook, arguments):
         bin_path = self.get_cli_hook(hook)
-        fn = self.makeFile("#!/bin/sh\n'%s' %s" % (bin_path, arguments))
+        fn = self.makeFile("#!/bin/bash\n'%s' %s" % (bin_path, arguments))
         # make the hook executable
         os.chmod(fn, stat.S_IEXEC | stat.S_IREAD)
         return fn
 
+    def create_capturing_hook(self, hook, files=()):
+        """Create a hook to enable capturing of results into files.
+
+        This method helps test scenarios of bash scripts that depend
+        on exact captures of stdout and stderr as well as set -eu
+        (Juju hook commands do not return nonzero exit codes, except
+        in the case of parse failures).
+
+        bin-path (the path to Juju commands) is always defined, along
+        with paths to temporary files, as keyed to the list of
+        `files`.
+        """
+        args = {"bin-path": os.path.normpath(
+                os.path.join(get_module_directory(juju), "..", "bin"))}
+        for f in files:
+            args[f] = self.makeFile()
+        hook_fn = self.makeFile(hook.format(**args))
+        os.chmod(hook_fn, stat.S_IEXEC | stat.S_IREAD)
+        return hook_fn, args
+
+    def assert_file(self, path, data):
+        """Assert file reached by `path` contains exactly this `data`."""
+        with open(path) as f:
+            self.assertEqual(f.read(), data)
+
 
 class TestCompleteInvoker(InvokerTestBase, StatusTestBase):
 
@@ -554,7 +595,7 @@ class InvokerTest(RelationInvokerTestBase):
         # ERROR indicate the level name, we are checking that the
         # proper level was logged here
         yield exe.ended
-        self.assertIn("ERROR", self.log.getvalue())
+        self.assertIn("INFO", self.log.getvalue())
         # and the message
         self.assertIn("FAIL", self.log.getvalue())
         self.assertIn("exit code 1", self.log.getvalue())
@@ -727,7 +768,7 @@ class InvokerTest(RelationInvokerTestBase):
         zk_data = yield self.relation.get_data()
         self.assertEqual(
             {"a": "b", "c": "d", "private-address": "mysql-0.example.com"},
-            yaml.load(zk_data))
+            serializer.load(zk_data))
         yield exe.ended
         self.assertIn(
             "Flushed values for hook %r on 'database:42'\n"
@@ -770,7 +811,7 @@ class InvokerTest(RelationInvokerTestBase):
         self.assertEqual(
             {"new-value": "2", "changed": "abc", "changed2": "xyz",
              "private-address": "mysql-0.example.com"},
-            yaml.load(zk_data))
+            serializer.load(zk_data))
 
         # Verify that unicode/strings longer than 100 characters in
         # representation (including quotes and the u marker) are cut
@@ -815,7 +856,7 @@ class InvokerTest(RelationInvokerTestBase):
         zk_data = yield self.relation.get_data()
         self.assertEqual({"no-change": "42", "untouched": "xyz",
                           "private-address": "mysql-0.example.com"},
-                         yaml.load(zk_data))
+                         serializer.load(zk_data))
         self.assertNotIn(
             "Flushed values for hook 'set-does-nothing'",
             output.getvalue())
@@ -959,7 +1000,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         internal_unit_id = (yield context.get_local_unit_state()).internal_id
         path = yield context.get_settings_path(internal_unit_id)
         data, stat = yield self.client.get(path)
-        self.assertEqual(yaml.load(data), expected)
+        self.assertEqual(serializer.load(data), expected)
 
     @defer.inlineCallbacks
     def test_implied_relation_hook_context(self):
@@ -1209,6 +1250,29 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
              "Relation name must be specified"),
             hook_log.getvalue())
 
+    @defer.inlineCallbacks
+    def test_relation_ids_nonexistent_relation_name(self):
+        """Verify an empty listing is returned if name does not exist"""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-ids does-not-exist 2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stdout", "stderr"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"], "")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
     @defer.inlineCallbacks
     def test_relation_set_with_relation_id_option(self):
         """Verify `relation-set` works with -r option."""
@@ -1236,6 +1300,57 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
              "    Setting changed: u'a'=u'42' (was unset) on 'db:1'",
              "    Setting changed: u'b'=u'xyz' (was unset) on 'db:1'"])
 
+    @defer.inlineCallbacks
+    def test_relation_set_with_nonexistent_relation_id(self):
+        """Verify error put on stderr when using nonexistent relation id."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation,
+            client_id="client_id")
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-set -r db:12345 "
+            "k1=v1 k2=v2 2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stdout", "stderr"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
+    @defer.inlineCallbacks
+    def test_relation_set_with_invalid_relation_id(self):
+        """Verify message is written to stderr for invalid formatted id."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        exe.environment["JUJU_REMOTE_UNIT"] = "wordpress/0"
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-set -r invalid-id:forty-two "
+            "k1=v1 k2=v2 2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stderr", "stdout"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"],
+                         "Not a valid relation id: 'invalid-id:forty-two'\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
     @defer.inlineCallbacks
     def test_relation_get_with_relation_id_option(self):
         """Verify `relation-get` works with -r option."""
@@ -1262,6 +1377,81 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
             {"private-address": "mysql-0.example.com",
              "name": "whiterabbit"})
 
+    @defer.inlineCallbacks
+    def test_relation_get_with_nonexistent_relation_id(self):
+        """Verify error put on stderr when using nonexistent relation id."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation,
+            client_id="client_id")
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "settings=$({bin-path}/relation-get -r db:12345 - mysql/0 "
+            "2> {stderr})\n"
+            "echo -n $settings > {settings}\n",
+            files=["settings", "stderr"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["settings"], "")
+        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
+    @defer.inlineCallbacks
+    def test_relation_get_with_invalid_relation_id(self):
+        """Verify message is written to stderr for invalid formatted id."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        exe.environment["JUJU_REMOTE_UNIT"] = "wordpress/0"
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-get -r invalid-id:forty-two - "
+            "2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stderr", "stdout"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stderr"],
+                         "Not a valid relation id: 'invalid-id:forty-two'\n")
+        self.assert_file(args["stdout"], "")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
+    @defer.inlineCallbacks
+    def test_relation_get_unset_remote_unit_in_env(self):
+        """Verify error is reported if JUJU_REMOTE_UNIT is not defined."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        self.assertNotIn("JUJU_REMOTE_UNIT", exe.environment)
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-get -r db:0 - 2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stderr", "stdout"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"], "Unit name is not defined\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
     @defer.inlineCallbacks
     def test_relation_list_with_relation_id_option(self):
         """Verify `relation-list` works with -r option."""
@@ -1282,6 +1472,55 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
             json.loads(hook_log.getvalue()),
             ["wordpress2/0"])
 
+    @defer.inlineCallbacks
+    def test_relation_list_with_nonexistent_relation_id(self):
+        """Verify a nonexistent relation id writes message to stderr."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-list -r db:12345 2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stdout", "stderr"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
+    @defer.inlineCallbacks
+    def test_relation_list_with_invalid_relation_id(self):
+        """Verify message is written to stderr for invalid formatted id."""
+        hook_log = self.capture_logging("hook", level=logging.DEBUG)
+        yield self.add_a_blog("wordpress2")
+        exe = yield self.ua.get_invoker(
+            "db:0", "add", "mysql/0", self.relation, client_id="client_id")
+        exe.environment["JUJU_REMOTE_UNIT"] = "wordpress/0"
+        hook, args = self.create_capturing_hook(
+            "#!/bin/bash\n"
+            "set -eu\n"
+            "data=$({bin-path}/relation-list -r invalid-id:forty-two "
+            "2> {stderr})\n"
+            "echo -n $data > {stdout}\n",
+            files=["stderr", "stdout"])
+        result = yield exe(hook)
+        self.assertEqual(result, 0)
+        yield exe.ended
+        self.assert_file(args["stdout"], "")
+        self.assert_file(args["stderr"],
+                         "Not a valid relation id: 'invalid-id:forty-two'\n")
+        self.assertEqual(
+            hook_log.getvalue(),
+            "Cached relation hook contexts on 'db:0': ['db:1']\n"
+            "hook {} exited, exit code 0.\n".format(os.path.basename(hook)))
+
 
 class PortCommandsTest(RelationInvokerTestBase):
 
@@ -1477,13 +1716,15 @@ class SubordinateRelationInvokerTest(InvokerTestBase, RelationTestBase):
 
         mu1, mu2 = my_units
         lu1, lu2 = log_units
+        self.mysql_units = my_units
+        self.log_units = log_units
 
         mystate = pick_attr(service_states, relation_role="server")
         logstate = pick_attr(service_states, relation_role="client")
 
         yield mystate.add_unit_state(mu1)
         self.relation = yield logstate.add_unit_state(lu1)
-        # add the second container (
+        # add the second container
         yield mystate.add_unit_state(mu2)
         self.relation2 = yield logstate.add_unit_state(lu2)
 
@@ -1509,24 +1750,56 @@ class SubordinateRelationInvokerTest(InvokerTestBase, RelationTestBase):
                                             "--format=smart"))
         self.assertEqual(result, 0)
         # verify that we see the proper unit
+        yield exe.ended
         self.assertIn("mysql/1", self.log.getvalue())
         # we don't see units in the other container
         self.assertNotIn("mysql/0", self.log.getvalue())
 
+    @defer.inlineCallbacks
+    def test_open_and_close_ports(self):
+        """Verify that port hook commands run and changes are immediate."""
+        unit_state = self.log_units[0]
+        container_state = self.mysql_units[0]
 
-def capture_separate_log(name, level):
-    """Support the separate capture of logging at different log levels.
+        self.assertEqual((yield unit_state.get_open_ports()), [])
 
-    TestCase.capture_logging only allows one level to be captured at
-    any given time. Given that the hook log captures both data AND
-    traditional logging, it's useful to separate.
-    """
-    logger = logging.getLogger(name)
-    output = StringIO()
-    handler = logging.StreamHandler(output)
-    handler.setLevel(level)
-    logger.addHandler(handler)
-    return output
+        exe = yield self.ua.get_invoker(
+            "database:42", "add", "logging/0", self.relation)
+        result = yield exe(self.create_hook("open-port", "80"))
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            (yield unit_state.get_open_ports()),
+            [{"port": 80, "proto": "tcp"}])
+        self.assertEqual(
+            (yield container_state.get_open_ports()),
+            [{"port": 80, "proto": "tcp"}])
+
+        result = yield exe(self.create_hook("open-port", "53/udp"))
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            (yield unit_state.get_open_ports()),
+            [{"port": 80, "proto": "tcp"},
+             {"port": 53, "proto": "udp"}])
+        self.assertEqual(
+            (yield container_state.get_open_ports()),
+            [{"port": 80, "proto": "tcp"},
+             {"port": 53, "proto": "udp"}])
+
+        result = yield exe(self.create_hook("close-port", "80/tcp"))
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            (yield unit_state.get_open_ports()),
+            [{"port": 53, "proto": "udp"}])
+        self.assertEqual(
+            (yield container_state.get_open_ports()),
+            [{"port": 53, "proto": "udp"}])
+
+        yield exe.ended
+        self.assertLogLines(
+            self.log.getvalue(), [
+                "opened 80/tcp",
+                "opened 53/udp",
+                "closed 80/tcp"])
 
 
 class TestCharmFormatV1(RelationInvokerTestBase):
@@ -1591,6 +1864,7 @@ class TestCharmFormatV1(RelationInvokerTestBase):
         yield exe(self.create_hook(
                 "relation-set", "d=@%s" % data_file))
         result = yield exe(self.create_hook("relation-get", "d mysql/0"))
+        self.assertEqual(result, 1)
         self.assertIn(
             "Error: \'utf8\' codec can\'t decode byte 0xca in position 30: "
             "invalid continuation byte",
@@ -1607,6 +1881,16 @@ class TestCharmFormatV2(RelationInvokerTestBase):
             "mysql", charm_name="mysql-format-v2")
         yield super(TestCharmFormatV2, self)._default_relations()
 
+    def make_zipped_file(self):
+        data_file = self.makeFile()
+        with open(data_file, "wb") as f:
+            # gzipped of 'abc' - however gzip will also includes the
+            # source file name, so easiest to keep it stable here as
+            # standard data
+            f.write("\x1f\x8b\x08\x08\xbb\x8bAP\x02\xfftmpr"
+                    "fyP0e\x00KLJ\x06\x00\xc2A$5\x03\x00\x00\x00")
+        return data_file
+
     @defer.inlineCallbacks
     def test_environment(self):
         """Ensure that an explicit setting of format: 2 works properly"""
@@ -1616,7 +1900,7 @@ class TestCharmFormatV2(RelationInvokerTestBase):
         self.assertEqual(env["_JUJU_CHARM_FORMAT"], "2")
 
     @defer.inlineCallbacks
-    def test_output(self):
+    def test_smart_output(self):
         """Verify roundtripping"""
         hook_debug_log = capture_separate_log("hook", level=logging.DEBUG)
         hook_log = capture_separate_log("hook", level=logging.INFO)
@@ -1624,54 +1908,235 @@ class TestCharmFormatV2(RelationInvokerTestBase):
             "database:42", "add", "mysql/0", self.relation,
             client_id="client_id")
 
-        # Byte strings are also supported by staying completely in
-        # YAML, so test that corner case. This also means users need
-        # to present valid YAML for any input:
-        data_file = self.makeFile(
-            yaml.safe_dump("But when I do drink, I prefer \xCA\xFE"))
+        # Test the support of raw strings, both from a file and from
+        # command line. Unicode can also be used - this is just
+        # rendered as UTF-8 in the shell; the source here is also
+        # UTF-8 - note it is not a Unicode string, it's a bytestring.
+        data_file = self.make_zipped_file()
         set_hook = self.create_hook(
             "relation-set",
-            "b=true i=42 f=1.23 s=ascii u=中文 d=@%s" % data_file)
+            "b=true f=1.23 i=42 s=ascii u=中文 d=@%s "
+            "r=\"$(echo -en 'But when I do drink, I prefer \\xCA\\xFE')\"" % (
+                data_file))
         yield exe(set_hook)
+
         result = yield exe(self.create_hook("relation-get", "- mysql/0"))
         self.assertEqual(result, 0)
 
-        # YAML guarantees that the keys will be sorted
-        # lexicographically; note that we output UTF-8 for Unicode
-        # when dumping YAML, so our source text (with this test file
-        # in UTF-8 itself) matches the output text, as seen in the
-        # characters for "zhongwen" (Chinese language).
+        # relation-get - uses YAML to dump keys. YAML guarantees that
+        # the keys will be sorted lexicographically; note that we
+        # output UTF-8 for Unicode when dumping YAML, so our source
+        # text (with this test file in UTF-8 itself) matches the
+        # output text, as seen in the characters for "zhongwen"
+        # (Chinese language).
         self.assertEqual(
             hook_log.getvalue(),
-            "b: true\n"
-            "d: !!binary |\n    QnV0IHdoZW4gSSBkbyBkcmluaywgSSBwcmVmZXIgyv4=\n"
-            "f: 1.23\n"
-            "i: 42\n"
+            "b: 'true'\n"
+            "d: !!binary |\n    H4sICLuLQVAC/3RtcHJmeVAwZQBLTEoGAMJBJDUDAAAA\n"
+            "f: '1.23'\n"
+            "i: '42'\n"
             "private-address: mysql-0.example.com\n"
+            "r: !!binary |\n    QnV0IHdoZW4gSSBkbyBkcmluaywgSSBwcmVmZXIgyv4=\n"
             "s: ascii\n"
-            "u: 中文\n\n")
+            "u: 中文\n")
 
-        # Log lines are not simply converted into Unicode, as in v1 format
+        # Note: backslashes are necessarily doubled here; r"XYZ"
+        # strings don't help with hexescapes
         self.assertLogLines(
             hook_debug_log.getvalue(),
             ["Flushed values for hook %r on 'database:42'" % (
                     os.path.basename(set_hook),),
-             "    Setting changed: 'b'=True (was unset)",
-             "    Setting changed: 'd'='But when I do drink, "
+             "    Setting changed: 'b'='true' (was unset)",
+             "    Setting changed: 'd'='\\x1f\\x8b\\x08\\x08\\xbb\\x8bAP\\x02"
+             "\\xfftmprfyP0e\\x00KLJ\\x06\\x00\\xc2A$5\\x03"
+             "\\x00\\x00\\x00' (was unset)",
+             "    Setting changed: 'f'='1.23' (was unset)",
+             "    Setting changed: 'i'='42' (was unset)",
+             "    Setting changed: 'r'='But when I do drink, "
              "I prefer \\xca\\xfe' (was unset)",
-             "    Setting changed: 'f'=1.23 (was unset)",
-             "    Setting changed: 'i'=42 (was unset)",
              "    Setting changed: 's'='ascii' (was unset)",
-             "    Setting changed: 'u'=u'\\u4e2d\\u6587' (was unset)"])
+             "    Setting changed: 'u'=u'\\u4e2d\\u6587' (was unset)"
+             ])
 
-        # Also ensure that invalid YAML is rejected; unlike earlier,
-        # this was not encoded with yaml.safe_dump
-        data_file = self.makeFile(
-            "But when I do drink, I prefer \xCA\xFE")
-        hook = self.create_hook("relation-set", "d=@%s" % data_file)
-        e = yield self.assertFailure(exe(hook), errors.CharmInvocationError)
-        self.assertEqual(str(e), "Error processing %r: exit code 1." % hook)
-        self.assertIn(
-            "yaml.reader.ReaderError: \'utf8\' codec can\'t decode byte #xca: "
-            "invalid continuation byte\n  in \"<string>\", position 30",
-            hook_log.getvalue())
+    @defer.inlineCallbacks
+    def test_exact_roundtrip_binary_data(self):
+        """Verify that binary data, including \x00, is roundtripped exactly"""
+        hook_log = capture_separate_log("hook", level=logging.INFO)
+        exe = yield self.ua.get_invoker(
+            "database:42", "add", "mysql/0", self.relation,
+            client_id="client_id")
+        data_file = self.make_zipped_file()
+
+        # relation-set can only read null bytes from a file; bash
+        # would otherwise silently drop
+        set_hook = self.create_hook("relation-set", "zipped=@%s" % (
+                data_file))
+        result = yield exe(set_hook)
+        self.assertEqual(result, 0)
+
+        # Abuse the create_hook method a little bit by adding a pipe
+        get_hook = self.create_hook("relation-get", "zipped mysql/0 | zcat")
+        result = yield exe(get_hook)
+        self.assertEqual(result, 0)
+
+        # Using the hook log for this verification does generate one
+        # extra \n (seen elsewhere in our tests), but this is just
+        # test noise: we are guaranteed roundtrip fidelity by using
+        # the picky tool that is zcat - no extraneous data accepted.
+        self.assertEqual(hook_log.getvalue(), "abc\n")
+
+    @defer.inlineCallbacks
+    def test_json_output(self):
+        """Verify roundtripping"""
+        hook_log = capture_separate_log("hook", level=logging.INFO)
+        exe = yield self.ua.get_invoker(
+            "database:42", "add", "mysql/0", self.relation,
+            client_id="client_id")
+
+        # Test the support of raw strings, both from a file and from
+        # command line. In addition, test Unicode indirectly by using
+        # UTF-8. Because the source of this file is marked as UTF-8,
+        # we can embed such characters directly in bytestrings, not
+        # just Unicode strings. This also works within the context of
+        # the shell.
+        raw = "But when I do drink, I prefer \xca\xfe"
+        data_file = self.makeFile(raw)
+        set_hook = self.create_hook(
+            "relation-set",
+            "b=true f=1.23 i=42 s=ascii u=中文 d=@%s "
+            "r=\"$(echo -en 'But when I do drink, I prefer \\xCA\\xFE')\"" % (
+                data_file,))
+        yield exe(set_hook)
+
+        result = yield exe(self.create_hook(
+                "relation-get", "--format=json - mysql/0"))
+        self.assertEqual(result, 0)
+
+        # YAML serialization internally has converted (transparently)
+        # UTF-8 to Unicode, which can be rendered by JSON. However the
+        # "cafe" bytestring is invalid JSON, so verify that it's been
+        # Base64 encoded.
+        encoded = base64.b64encode(raw)
+        self.assertEqual(
+            hook_log.getvalue(),
+            '{"b": "true", '
+            '"d": "%s", '
+            '"f": "1.23", '
+            '"i": "42", '
+            '"private-address": "mysql-0.example.com", '
+            '"s": "ascii", '
+            '"r": "%s", '
+            '"u": "\\u4e2d\\u6587"}\n' % (encoded, encoded))
+
+    @defer.inlineCallbacks
+    def common_relation_set(self):
+        hook_log = capture_separate_log("hook", level=logging.INFO)
+        exe = yield self.ua.get_invoker(
+            "database:42", "add", "mysql/0",
+            self.relation, client_id="client_id")
+        raw = "But when I do drink, I prefer \xCA\xFE"
+        data_file = self.makeFile(raw)
+        set_hook = self.create_hook(
+            "relation-set",
+            "s='some text' u=中文 d=@%s "
+            "r=\"$(echo -en 'But when I do drink, I prefer \\xCA\\xFE')\"" % (
+                data_file))
+        result = yield exe(set_hook)
+        self.assertEqual(result, 0)
+        defer.returnValue((exe, hook_log))
+
+    @defer.inlineCallbacks
+    def test_relation_get_ascii(self):
+        """Verify that ascii data is roundtripped"""
+        exe, hook_log = yield self.common_relation_set()
+        result = yield exe(self.create_hook("relation-get", "s mysql/0"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "some text\n")
+
+    @defer.inlineCallbacks
+    def test_relation_get_raw(self):
+        """Verify that raw data is roundtripped"""
+        exe, hook_log = yield self.common_relation_set()
+        result = yield exe(self.create_hook("relation-get", "r mysql/0"))
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            hook_log.getvalue(), "But when I do drink, I prefer \xca\xfe\n")
+
+    @defer.inlineCallbacks
+    def test_relation_get_unicode(self):
+        """Verify Unicode is roundtripped (via UTF-8) through the shell"""
+        exe, hook_log = yield self.common_relation_set()
+
+        result = yield exe(self.create_hook("relation-get", "u mysql/0"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "中文\n")
+
+    @defer.inlineCallbacks
+    def setup_config(self):
+        hook_log = self.capture_logging("hook")
+        exe = yield self.ua.get_invoker(
+            "db:42", "add", "mysql/0", self.relation, client_id="client_id")
+        context = yield self.ua.get_context("client_id")
+        config = yield context.get_config()
+        with open(self.make_zipped_file(), "rb") as f:
+            data = f.read()
+        config.update({
+            "b": True,
+            "f": 1.23,
+            "i": 42,
+            "s": "some text",
+            # uses UTF-8 encoding in this test script
+            "u": "中文",
+            # use high byte and null byte characters
+            "r": data
+            })
+        yield config.write()
+        defer.returnValue((exe, hook_log))
+
+    @defer.inlineCallbacks
+    def test_config_get_boolean(self):
+        """Validate that config-get returns lowercase names of booleans."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "b"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "true\n")
+
+    @defer.inlineCallbacks
+    def test_config_get_float(self):
+        """Validate that config-get returns floats without quotes."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "f"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "1.23\n")
+
+    @defer.inlineCallbacks
+    def test_config_get_int(self):
+        """Validate that config-get returns ints without quotes."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "i"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "42\n")
+
+    @defer.inlineCallbacks
+    def test_config_get_ascii(self):
+        """Validate that config-get returns ascii strings."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "s"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "some text\n")
+
+    @defer.inlineCallbacks
+    def test_config_get_raw(self):
+        """Validate config-get can work with high and null bytes."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "r | zcat"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "abc\n")
+
+    @defer.inlineCallbacks
+    def test_config_get_unicode(self):
+        """Validate that config-get returns raw strings containing UTF-8."""
+        exe, hook_log = yield self.setup_config()
+        result = yield exe(self.create_hook("config-get", "u"))
+        self.assertEqual(result, 0)
+        self.assertEqual(hook_log.getvalue(), "中文\n")
