@@ -1,9 +1,10 @@
 from base64 import b64encode
 from subprocess import Popen, PIPE
-from yaml import safe_dump
+
 
 from juju.errors import CloudInitError
 from juju.lib.upstart import UpstartService
+from juju.lib import serializer
 from juju.providers.common.utils import format_cloud_init
 from juju.state.auth import make_identity
 import juju
@@ -18,7 +19,7 @@ def _branch_install_scripts(branch):
     return [
         "sudo apt-get install -y python-txzookeeper",
         "sudo mkdir -p /usr/lib/juju",
-        "cd /usr/lib/juju && sudo /usr/bin/bzr co %s juju" % branch,
+        "cd /usr/lib/juju && sudo /usr/bin/bzr co --lightweight %s juju" % branch,
         "cd /usr/lib/juju/juju && sudo python setup.py develop"]
 
 
@@ -41,7 +42,7 @@ def _zookeeper_scripts(instance_id, secret, constraints, provider_type):
         " --constraints-data=%s"
         " --provider-type=%s"
         % (instance_id, make_identity("admin:%s" % secret),
-           b64encode(safe_dump(constraints.data)), provider_type)]
+           b64encode(serializer.dump(constraints.data)), provider_type)]
 
 
 def _machine_scripts(machine_id, zookeeper_hosts):
@@ -55,6 +56,27 @@ def _machine_scripts(machine_id, zookeeper_hosts):
         "--session-file /var/run/juju/machine-agent.zksession")
     return service.get_cloud_init_commands()
 
+def _unit_scripts(machine_id, unit_name, zookeeper_hosts, juju_home):
+    unit_path_name = unit_name.replace('/', '-')
+    service_name = "juju-%s" % unit_path_name
+    service = UpstartService(service_name)
+    service.set_description(
+        "Juju unit agent for %s" % unit_name)
+    service.set_environ(
+            {"JUJU_MACHINE_ID": str(machine_id),
+             "JUJU_UNIT_NAME": unit_name,
+             "JUJU_HOME": juju_home,
+             "JUJU_ZOOKEEPER": zookeeper_hosts})
+    service.set_output_path(
+        "/var/log/juju/unit-%s-output.log" % unit_path_name)
+    service.set_command(" ".join(
+                ["/usr/bin/python",
+                "-m", "juju.agents.unit",
+                "--nodaemon",
+                "--logfile", "/var/log/juju/unit-%s.log" % unit_path_name,
+                "--session-file",
+                "/var/run/juju/unit-%s-agent.zksession" % unit_path_name]))
+    return service.get_cloud_init_commands()
 
 def _provision_scripts(zookeeper_hosts):
     service = UpstartService("juju-provision-agent")
@@ -107,7 +129,7 @@ def parse_juju_origin(data):
         # See if one of the sources is the PPA
         first_indent, line = next()
         while True:
-            if "http://ppa.launchpad.net/juju/pkgs/" in line:
+            if "http://ppa.launchpad.net/juju/0.6/" in line:
                 return PPA, None
             indent, line = next()
             if indent != first_indent:
@@ -150,6 +172,9 @@ class CloudInit(object):
         self._zookeeper_secret = None
         self._constraints = None
         self._origin, self._origin_url = get_default_origin()
+        self._unit_name = None
+        self._juju_home = None
+        self._apt_proxy = None
 
     def add_ssh_key(self, key):
         """Add an SSH public key.
@@ -212,6 +237,23 @@ class CloudInit(object):
         """
         self._machine_id = id
 
+    def set_unit_name(self, name):
+        """Specify the juju unit name.
+        :param str name: the desired Unit name.
+
+        This is optional, if present, will install a unit agent
+        """
+        self._unit_name = name
+
+    def set_juju_home(self, juju_home):
+        """Specify the juju unit home dir
+        :param str juju_home: the desired Unit home dir
+
+        This is required if _unit_name is set. Tells unit
+        agent where its home directory is.
+        """
+        self._juju_home = juju_home
+
     def set_instance_id_accessor(self, expr):
         """Specify the provider-specific instance ID.
 
@@ -242,6 +284,16 @@ class CloudInit(object):
         """
         self._zookeeper_hosts = [m.private_dns_name for m in machines]
 
+    def set_zookeeper_hosts(self, hosts):
+        """Specify master :class:`juju.machine.ProviderMachine` instances.
+
+        :param hosts: hosts running zookeeper which already exist.
+        :type list: list of str
+
+        This is an alternative to set_zookeeper_machines.
+        """
+        self._zookeeper_hosts = hosts
+
     def set_zookeeper_secret(self, secret):
         """Specify the admin password for zookeepeer.
 
@@ -249,6 +301,14 @@ class CloudInit(object):
         """
         self._zookeeper_secret = secret
 
+    def set_apt_proxy(self, proxy):
+        """Specify an apt proxy to configure
+        
+        :param proxy: proxy to set for APT using Acquire:HTTP
+        :type str
+        """
+        self._apt_proxy = proxy
+
     def set_constraints(self, constraints):
         """Specify the initial machine's constraints.
 
@@ -270,7 +330,8 @@ class CloudInit(object):
             packages=self._collect_packages(),
             repositories=self._collect_repositories(),
             scripts=self._collect_scripts(),
-            data=self._collect_machine_data())
+            data=self._collect_machine_data(),
+            apt_proxy=self._apt_proxy)
 
     def _validate(self):
         missing = []
@@ -280,9 +341,8 @@ class CloudInit(object):
                 missing.append(action)
 
         require("_ssh_keys", "add_ssh_key")
-        require("_machine_id", "set_machine_id")
-        require("_provider_type", "set_provider_type")
         if self._zookeeper:
+            require("_provider_type", "set_provider_type")
             require("_instance_id", "set_instance_id_accessor")
             require("_zookeeper_secret", "set_zookeeper_secret")
             require("_constraints", "set_constraints")
@@ -296,7 +356,13 @@ class CloudInit(object):
         all_hosts = self._zookeeper_hosts[:]
         if self._zookeeper:
             all_hosts.append("localhost")
-        return ",".join(["%s:2181" % host for host in all_hosts])
+        zks = []
+        for host in all_hosts:
+            if isinstance(host, tuple) and len(host) == 2:
+                zks.append("%s:%s" % host)
+            else:
+                zks.append("%s:2181" % host)
+        return ",".join(zks)
 
     def _collect_packages(self):
         packages = [
@@ -313,7 +379,7 @@ class CloudInit(object):
         if self._origin == PROPOSED:
             return ["deb $MIRROR $RELEASE-proposed main universe"]
         if self._origin != DISTRO:
-            return ["ppa:juju/pkgs"]
+            return ["ppa:juju/0.6"]
         return []
 
     def _collect_scripts(self):
@@ -324,8 +390,15 @@ class CloudInit(object):
                 self._zookeeper_secret,
                 self._constraints,
                 self._provider_type))
-        scripts.extend(_machine_scripts(
-            self._machine_id, self._join_zookeeper_hosts()))
+        if self._machine_id:
+            scripts.extend(_machine_scripts(
+                self._machine_id, self._join_zookeeper_hosts()))
+        if self._unit_name:
+            scripts.extend(_unit_scripts(
+                self._machine_id,
+                self._unit_name,
+                self._join_zookeeper_hosts(),
+                self._juju_home))
         if self._provision:
             scripts.extend(_provision_scripts(self._join_zookeeper_hosts()))
         return scripts
