@@ -1,14 +1,15 @@
 import os
 import stat
 
-import yaml
 
 from juju.errors import CloudInitError
+from juju.lib import serializer
 from juju.lib.testing import TestCase
 from juju.machine.tests.test_constraints import dummy_cs
 from juju.providers.common.cloudinit import (
     CloudInit, parse_juju_origin, get_default_origin)
 from juju.providers.dummy import DummyMachine
+from juju.machine import ProviderMachine
 import juju
 
 
@@ -46,10 +47,10 @@ class CloudInitTest(TestCase):
 
     def assert_render(self, cloud_init, name):
         with open(os.path.join(DATA_DIR, name)) as f:
-            expected = yaml.load(f.read())
+            expected = serializer.load(f.read())
         rendered = cloud_init.render()
         self.assertTrue(rendered.startswith("#cloud-config"))
-        self.assertEquals(yaml.load(rendered), expected)
+        self.assertEquals(serializer.load(rendered), expected)
 
     def test_render_validate_normal(self):
         cloud_init = CloudInit()
@@ -57,7 +58,7 @@ class CloudInitTest(TestCase):
         self.assertEquals(
             str(error),
             "Incomplete cloud-init: you need to call add_ssh_key, "
-            "set_machine_id, set_provider_type, set_zookeeper_machines")
+            "set_zookeeper_machines")
 
     def test_render_validate_bootstrap(self):
         cloud_init = CloudInit()
@@ -66,7 +67,7 @@ class CloudInitTest(TestCase):
         self.assertEquals(
             str(error),
             "Incomplete cloud-init: you need to call add_ssh_key, "
-            "set_machine_id, set_provider_type, set_instance_id_accessor, "
+            "set_provider_type, set_instance_id_accessor, "
             "set_zookeeper_secret, set_constraints")
 
     def test_source_validate(self):
@@ -138,6 +139,39 @@ class CloudInitTest(TestCase):
         self.assert_render(
             self.construct_bootstrap(True), "cloud_init_bootstrap_zookeepers")
 
+    def test_render_no_machine_id(self):
+        self.patch(juju, "__file__", "/not/installed/under/usr")
+        cloud_init = CloudInit()
+        cloud_init.add_ssh_key("chubb")
+        cloud_init.set_provider_type("dummy")
+        cloud_init.set_zookeeper_machines([
+            DummyMachine("blah", "blah", "cotswold"),
+            DummyMachine("blah", "blah", "longleat")])
+        cloud_init.set_juju_source(distro=True)
+        self.assert_render(cloud_init, "cloud_init_no_machine_id")
+
+    def test_render_apt_proxy(self):
+        cloud_init = self.construct_normal()
+        cloud_init.set_juju_source(ppa=True)
+        cloud_init.set_apt_proxy('superproxy:37337')
+        self.assert_render(cloud_init, "cloud_init_ppa_apt_proxy")
+
+    def test_set_zookeeper_machines(self):
+        machine1 = ProviderMachine('i-100', 'foo1', 'foo1.private')
+        machine2 = ProviderMachine('i-200', 'foo2', 'foo2.private')
+        cloud_init = CloudInit()
+        cloud_init.set_zookeeper_machines([machine1, machine2])
+        self.assertEqual(['foo1.private', 'foo2.private'], cloud_init._zookeeper_hosts)
+
+    def test_set_zookeeper_hosts(self):
+        cloud_init = CloudInit()
+        cloud_init.set_zookeeper_hosts(['bar1.private', 'bar2.private'])
+        self.assertEqual(['bar1.private', 'bar2.private'], cloud_init._zookeeper_hosts)
+
+    def test_set_zookeeper_hosts_tuple(self):
+        cloud_init = CloudInit()
+        cloud_init.set_zookeeper_hosts([('bar1.private', 9999)])
+        self.assertEqual('bar1.private:9999', cloud_init._join_zookeeper_hosts())
 
 class ParseJujuOriginTest(TestCase):
 
@@ -148,7 +182,7 @@ class ParseJujuOriginTest(TestCase):
             "  Candidate: good-magic-1.0\n"
             "  Version table:\n"
             "     0.5+bzr366-1juju1~natty1 0\n"
-            "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+            "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
             "natty/main amd64 Packages\n"
             " *** good-magic-1.0 0\n"
             "        500 http://us.archive.ubuntu.com/ubuntu/ "
@@ -170,7 +204,7 @@ class ParseJujuOriginTest(TestCase):
             "natty/main amd64 Packages\n"
             " *** 0.5+bzr366-1juju1~natty1 0\n"
             "        100 /var/lib/dpkg/status\n"
-            "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+            "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
             "natty/main amd64 Packages\n"
             "     0.5+bzr356-1juju1~natty1 0\n"
             "        500 http://us.archive.ubuntu.com/ubuntu/ "
@@ -188,7 +222,7 @@ class ParseJujuOriginTest(TestCase):
             "  Version table:\n"
             "     0.5+bzr366-1juju1~natty1 0\n"
             "        100 /var/lib/dpkg/status\n"
-            "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+            "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
             "natty/main amd64 Packages\n"
             "     good-magic-1.0 0\n"
             "        500 http://us.archive.ubuntu.com/ubuntu/ "
@@ -209,7 +243,7 @@ class ParseJujuOriginTest(TestCase):
             "        500 http://us.archive.ubuntu.com/ubuntu/ "
             "natty/main amd64 Packages\n"
             " *** 0.5+bzr356-1juju1~natty1 0\n"
-            "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+            "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
             "natty/main amd64 Packages\n"
             "        100 /var/lib/dpkg/status\n")
 
@@ -253,7 +287,7 @@ class ParseJujuOriginTest(TestCase):
                 "  Candidate: good-magic-1.0\n"
                 "  Version table:\n"
                 " *** good-magic-1.0 0\n"
-                "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+                "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
                 "natty/main amd64 Packages\n"
                 "        100 /var/lib/dpkg/status\n"
                 "EOF\n")
@@ -278,7 +312,7 @@ class ParseJujuOriginTest(TestCase):
                 "  Candidate: good-magic-1.0\n"
                 "  Version table:\n"
                 " *** good-magic-1.0 0\n"
-                "        500 http://ppa.launchpad.net/juju/pkgs/ubuntu/ "
+                "        500 http://ppa.launchpad.net/juju/0.6/ubuntu/ "
                 "natty/main amd64 Packages\n"
                 "        100 /var/lib/dpkg/status\n"
                 "EOF\n")
