@@ -42,8 +42,21 @@ def extract_system_id(resource_uri):
         return match.group("system_id")
 
 
+def _int_str(value):
+    """Convert value to integer then string"""
+    return str(int(value))
+
+
 class MAASClient(MAASOAuthConnection):
 
+    _handled_constraints = (
+        ("maas-name", "name", str),
+        ("maas-tags", "tags", str),
+        ("arch", "arch", str),
+        ("cpu", "cpu_count", _int_str),
+        ("mem", "mem", _int_str),
+        )
+
     def __init__(self, config):
         """Initialise an API client for MAAS.
 
@@ -119,12 +132,13 @@ class MAASClient(MAASOAuthConnection):
         """
         params = {"op": "acquire"}
         if constraints is not None:
-            name = constraints["maas-name"]
-            if name is not None:
-                params["name"] = name
+            for key_from, key_to, translate in self._handled_constraints:
+                value = constraints.get(key_from, None)
+                if value is not None:
+                    params[key_to] = translate(value)
         return self.post("api/1.0/nodes/", params)
 
-    def start_node(self, resource_uri, user_data):
+    def start_node(self, resource_uri, ubuntu_series, user_data):
         """Ask MAAS to start a node.
 
         :param resource_uri: The MAAS URI for the node you want to start.
@@ -135,7 +149,7 @@ class MAASClient(MAASOAuthConnection):
         """
         assert isinstance(user_data, str), (
             "User data must be a byte string.")
-        params = {"op": "start", "user_data": b64encode(user_data)}
+        params = {"op": "start", "distro_series": ubuntu_series, "user_data": b64encode(user_data)}
         return self.post(resource_uri, params)
 
     def stop_node(self, resource_uri):
@@ -157,3 +171,11 @@ class MAASClient(MAASOAuthConnection):
         """
         params = {"op": "release"}
         return self.post(resource_uri, params)
+
+    def list_tags(self):
+        """Ask MAAS to return a list of all the tags defined.
+
+        :return: A Deferred whose value is the list of tags.
+        """
+        params = {"op": "list"}
+        return self.get("api/1.0/tags/", params)
