@@ -6,7 +6,6 @@
 from StringIO import StringIO
 from twisted.internet import defer
 from twisted.internet.defer import inlineCallbacks
-from twisted.web.error import Error
 
 from juju.errors import ProviderError
 from juju.lib.mocker import ANY
@@ -72,7 +71,7 @@ class LaunchMachineTest(TestCase):
         provider = self._get_provider(
             FakeMAASHTTPConnectionWithNoAvailableNodes)
         machine_data = {
-            "machine-id": "foo", "constraints": {"maas-name": None}}
+            "machine-id": "foo", "constraints": {"ubuntu-series": "splendid"}}
         d = provider.start_machine(machine_data)
 
         # These arbitrary fake failure values come from
@@ -94,7 +93,7 @@ class LaunchMachineTest(TestCase):
         # Try to start up that node using the fake MAAS.
         provider = self._get_provider(FakeMAASHTTPConnection)
         machine_data = {
-            "machine-id": machine_id, "constraints": {"maas-name": None}}
+            "machine-id": "foo", "constraints": {"ubuntu-series": "splendid"}}
         machine_list = yield provider.start_machine(machine_data)
 
         # Test that it returns a list containing a single MAASMachine
@@ -118,7 +117,7 @@ class LaunchMachineTest(TestCase):
         self.assertEqual("moon", target_node["hostname"])
         machine_id = "foo"
         provider = self._get_provider(FakeMAASHTTPConnection)
-        constraints = {"maas-name": "moon"}
+        constraints = {"maas-name": "moon", "ubuntu-series": "splendid"}
         machine_data = {"machine-id": machine_id, "constraints": constraints}
         machine_list = yield provider.start_machine(machine_data)
 
@@ -147,10 +146,10 @@ class LaunchMachineTest(TestCase):
         # The following operations happen in sequence.
         mocker.order()
         # First, the node is acquired.
-        mock_client.acquire_node({"maas-name": None})
+        mock_client.acquire_node({"ubuntu-series": "precise"})
         mocker.result({"resource_uri": "/node/123"})
         # Second, the node is started. We simulate a failure at this stage.
-        mock_client.start_node("/node/123", ANY)
+        mock_client.start_node("/node/123", "precise", ANY)
         mocker.throw(ZeroDivisionError)
         # Last, the node is released.
         mock_client.release_node("/node/123")
@@ -158,5 +157,6 @@ class LaunchMachineTest(TestCase):
 
         mocker.replay()
         return self.assertFailure(
-            MAASLaunchMachine(mock_provider, {"maas-name": None}).run("fred"),
+            MAASLaunchMachine(mock_provider,
+                              {"ubuntu-series": "precise"}).run("fred"),
             ZeroDivisionError)
