@@ -222,9 +222,10 @@ class TestMAASClientWithTwisted(TestCase, TestMAASClientBase):
     @inlineCallbacks
     def test_start_node(self):
         resource_uri = NODE_JSON[0]["resource_uri"]
+        series = "splendid"
         data = "This is test data."
         client, log = self.get_client()
-        returned_data = yield client.start_node(resource_uri, data)
+        returned_data = yield client.start_node(resource_uri, series, data)
         self.assertEqual(returned_data, NODE_JSON[0])
         # Also make sure that the connection was passed the user_data in
         # the POST data.
@@ -242,7 +243,7 @@ class TestMAASClientWithTwisted(TestCase, TestMAASClientBase):
     @inlineCallbacks
     def test_start_node_connects_with_oauth_credentials(self):
         client, log = self.get_client()
-        yield client.start_node("foo", "bar")
+        yield client.start_node("foo", "splendid", "bar")
         [factory_call] = [
             record for record in log if record.called == "factory"]
         self.assertIn("Authorization", factory_call.result.headers)
@@ -336,3 +337,35 @@ class TestConstraints(TestCase, TestMAASClientBase):
         self.assertIs(None, guinness)
         name = client.post.params_used.get("name")
         self.assertEqual("zaphod", name)
+
+    def test_acquire_node_handles_arch_constraint(self):
+        client = self.set_up_client_with_fake()
+        constraints = {"arch": "i386"}
+        client.acquire_node(constraints)
+
+        arch = client.post.params_used.get("arch")
+        self.assertEqual("i386", arch)
+
+    def test_acquire_node_handles_cpu_constraint(self):
+        client = self.set_up_client_with_fake()
+        constraints = {"cpu": 2.0}
+        client.acquire_node(constraints)
+
+        cpu_count = client.post.params_used.get("cpu_count")
+        self.assertEqual("2", cpu_count)
+
+    def test_acquire_node_handles_mem_constraint(self):
+        client = self.set_up_client_with_fake()
+        constraints = {"mem": 2048.0}
+        client.acquire_node(constraints)
+
+        mem = client.post.params_used.get("mem")
+        self.assertEqual("2048", mem)
+
+    def test_acquire_node_handles_arbitrary_tag_query(self):
+        client = self.set_up_client_with_fake()
+        constraints = {"maas-tags": "red&!white|blue"}
+        client.acquire_node(constraints)
+
+        tags = client.post.params_used.get("tags")
+        self.assertEqual("red&!white|blue", tags)
