@@ -2,15 +2,14 @@ import logging
 import os
 import time
 
-import yaml
 import zookeeper
 
-
 from twisted.internet.defer import (
     inlineCallbacks, returnValue, Deferred, fail, succeed)
 
 from juju.charm.directory import CharmDirectory
 from juju.charm.tests import local_charm_id
+from juju.lib import serializer
 from juju.machine.tests.test_constraints import dummy_constraints
 from juju.charm.tests.test_metadata import test_repository_path
 from juju.charm.tests.test_repository import unbundled_repository
@@ -803,14 +802,14 @@ class ServiceRelationStateTest(RelationTestBase):
         # Verify the unit mapping
         unit_map_data, stat = yield self.client.get("/relations/%s" % (
             self.relation_state.internal_id))
-        unit_map = yaml.load(unit_map_data)
+        unit_map = serializer.load(unit_map_data)
         self.assertEqual(
             unit_map,
             {unit_state.internal_id: unit_state.unit_name})
 
         content, stat = yield self.client.get(settings_path)
         self.assertEqual(
-            yaml.load(content), {"private-address": "foobar.local"})
+            serializer.load(content), {"private-address": "foobar.local"})
 
     @inlineCallbacks
     def test_add_unit_state_scope_container_relation(self):
@@ -930,7 +929,7 @@ class ServiceRelationStateTest(RelationTestBase):
             unit_state.internal_id)
 
         data = {"hello": "world", "private-address": "foobar.local"}
-        yield self.client.create(settings_path, yaml.dump(data))
+        yield self.client.create(settings_path, serializer.dump(data))
 
         yield unit_state.set_private_address("northwest.local")
         yield self.service1_relation.add_unit_state(unit_state)
@@ -939,11 +938,11 @@ class ServiceRelationStateTest(RelationTestBase):
 
         # The unit address has been updated to current
         data["private-address"] = "northwest.local"
-        self.assertEqual(node_data, yaml.dump(data))
+        self.assertEqual(node_data, serializer.dump(data))
 
         data, stat = yield self.client.get(
             "/relations/%s" % self.relation_state.internal_id)
-        unit_map = yaml.load(data)
+        unit_map = serializer.load(data)
         self.assertEqual(unit_map,
                          {unit_state.internal_id: unit_state.unit_name})
 
@@ -1094,13 +1093,14 @@ class UnitRelationStateTest(RelationTestBase):
         unit_relation = states["unit_relation"]
 
         data = yield unit_relation.get_data()
-        self.assertEqual(yaml.load(data), {"private-address": None})
+        self.assertEqual(serializer.load(data), {"private-address": None})
 
         unit_relation_path = self.get_unit_settings_path(states)
-        self.client.set(unit_relation_path, yaml.dump(dict(hello="world")))
+        self.client.set(
+            unit_relation_path, serializer.dump(dict(hello="world")))
 
         data = yield unit_relation.get_data()
-        self.assertEqual(data, yaml.dump(dict(hello="world")))
+        self.assertEqual(data, serializer.dump(dict(hello="world")))
 
     @inlineCallbacks
     def test_set_data(self):
@@ -1109,7 +1109,7 @@ class UnitRelationStateTest(RelationTestBase):
         unit_relation_path = self.get_unit_settings_path(states)
         yield unit_relation.set_data(dict(hello="world"))
         data, stat = yield self.client.get(unit_relation_path)
-        self.assertEqual(data, yaml.dump(dict(hello="world")))
+        self.assertEqual(data, serializer.dump(dict(hello="world")))
 
     @inlineCallbacks
     def test_get_relation_role(self):
@@ -1137,7 +1137,7 @@ class UnitRelationStateTest(RelationTestBase):
         yield self.service_manager.remove_service_state(states["service"])
         yield self.assertFailure(
             states["unit_relation"].get_relation_role(), StateChanged)
-        
+
     @inlineCallbacks
     def test_get_related_unit_container(self):
         """Retrieve the container path of the related units."""
