@@ -8,8 +8,8 @@ from Queue import Queue
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks
 from twisted.internet.threads import deferToThread
-import yaml
 
+from juju.lib import serializer
 from juju.lib.testing import TestCase
 from juju.state.base import StateBase
 from juju.state.errors import StateChanged, StateNotFound
@@ -382,7 +382,7 @@ class YAMLStateTest(TestCase):
 
         # and a direct look at zk should work as well
         zk_data, stat = yield self.client.get(self.path)
-        zk_data = yaml.load(zk_data)
+        zk_data = serializer.load(zk_data)
         self.assertEqual(zk_data, options)
 
     @inlineCallbacks
@@ -465,7 +465,7 @@ class YAMLStateTest(TestCase):
 
         # and a direct look at zk should work as well
         zk_data, stat = yield self.client.get(self.path)
-        zk_data = yaml.load(zk_data)
+        zk_data = serializer.load(zk_data)
         self.assertEqual(zk_data, options)
 
     @inlineCallbacks
@@ -678,13 +678,14 @@ class YAMLStateNodeMixinTest(TestCase):
 
     @inlineCallbacks
     def test_get_missing(self):
-        yield self.client.create(self.path, yaml.dump({"foo": "bar"}))
+        yield self.client.create(self.path, serializer.dump({"foo": "bar"}))
         mixee = TestMixee(self.client, self.path)
         self.assertEquals((yield mixee.get()), None)
 
     @inlineCallbacks
     def test_get_exists(self):
-        yield self.client.create(self.path, yaml.dump({"key": "butterfly"}))
+        yield self.client.create(
+            self.path, serializer.dump({"key": "butterfly"}))
         mixee = TestMixee(self.client, self.path)
         self.assertEquals((yield mixee.get()), "butterfly")
 
@@ -694,13 +695,14 @@ class YAMLStateNodeMixinTest(TestCase):
         mixee = TestMixee(self.client, self.path)
         yield mixee.set("caterpillar")
         content, _ = yield self.client.get(self.path)
-        self.assertEquals(yaml.load(content), {"key": "caterpillar"})
+        self.assertEquals(serializer.load(content), {"key": "caterpillar"})
 
     @inlineCallbacks
     def test_set_safely(self):
-        yield self.client.create(self.path, yaml.dump({"foo": "bar"}))
+        yield self.client.create(self.path, serializer.dump({"foo": "bar"}))
         mixee = TestMixee(self.client, self.path)
         yield mixee.set("cocoon")
         content, _ = yield self.client.get(self.path)
-        self.assertEquals(yaml.load(content), {"foo": "bar", "key": "cocoon"})
+        self.assertEquals(
+            serializer.load(content), {"foo": "bar", "key": "cocoon"})
 
