@@ -1,13 +1,13 @@
-import yaml
+
 
 from juju.errors import IncompatibleVersion
-
+from juju.lib import serializer
 
 # The protocol version, which is stored in the /topology node under
 # the "version" key. The protocol version should *only* be updated
 # when we know that a version is in fact actually incompatible.
 
-VERSION = 1
+VERSION = 2
 
 
 class InternalTopologyError(Exception):
@@ -21,23 +21,26 @@ class InternalTopologyError(Exception):
 class InternalTopology(object):
     """Helper to deal with the high-level topology map stored in ZK.
 
-    This must not be used outside of juju.state.  To work with
-    the topology itself, check out the "machine" and "service" modules.
+    This must not be used outside of juju.state.  To work with the
+    topology itself, check out the "machine" and "service"modules.
 
-    The internal topology implementation is based on the use of single node to
-    function as a logical map of some entities within the zookeeper hierarchy.
-    Being a single node means that it may be changed atomically, and thus
-    the network of services have a central consistency point which may be
-    used to develop further algorithms on top of.  Without it, incomplete
-    creation of various multi-node objects would have to be considered.
+    The internal topology implementation is based on the use of single
+    node to function as a logical map of some entities within the
+    zookeeper hierarchy.  Being a single node means that it may be
+    changed atomically, and thus the network of services have a
+    central consistency point which may be used to develop further
+    algorithms on top of.  Without it, incomplete creation of various
+    multi-node objects would have to be considered.
 
     This topology contains details such as service names, service unit
     sequence numbers on a per service basis, mapping of service_id to
-    service names (service names are inteded as display names, and hence
-    subject to change), and mapping of service units to machines.
+    service names (service names are inteded as display names, and
+    hence subject to change), and mapping of service units to
+    machines.
+
+    The internal state is maintained in a dictionary, but its
+    structure and storage format should not be depended upon.
 
-    The internal state is maintained in a dictionary, but its structure and
-    storage format should not be depended upon.
     """
 
     _nil_dict = {}
@@ -56,15 +59,15 @@ class InternalTopology(object):
         This string may be provided to the :method:`parse` to
         reestablish the same topology state back.
         """
-        return yaml.safe_dump(self._state)
+        return serializer.dump(self._state)
 
     def parse(self, data):
-        """Parse the dumped data provided and restore the internal state.
+        """Parse the dumped data provided and restore internal state.
 
-        The provided data must necessarily have been retrieved by calling
-        the :method:`dump`.
+        The provided data must necessarily have been retrieved by
+        calling the :method:`dump`.
         """
-        parsed = yaml.load(data)
+        parsed = serializer.load(data)
         self._state = parsed
         version = self.get_version()
         if version != VERSION:
@@ -79,16 +82,18 @@ class InternalTopology(object):
         machines = self._state.setdefault("machines", {})
         if machine_id in machines:
             raise InternalTopologyError(
-                "Attempted to add duplicated machine (%s)" % machine_id)
+                "Attempted to add duplicated "
+                "machine (%s)" % machine_id)
         machines[machine_id] = {}
 
     def has_machine(self, machine_id):
         """Return True if machine_id was registered in the topology.
         """
-        return machine_id in self._state.get("machines", self._nil_dict)
+        return machine_id in self._state.get(
+            "machines", self._nil_dict)
 
     def get_machines(self):
-        """Return list of machine ids previously registered in this topology.
+        """Return list of machine ids registered in this topology.
         """
         return sorted(self._state.get("machines", self._nil_dict).keys())
 
@@ -131,9 +136,10 @@ class InternalTopology(object):
             unit_sequence[service_name] = 0
 
     def has_service(self, service_id):
-        """Return True if service_id was previously added to this topology.
+        """Return True if service_id was previously added.
         """
-        return service_id in self._state.get("services", self._nil_dict)
+        return service_id in self._state.get(
+            "services", self._nil_dict)
 
     def get_service_name(self, service_id):
         """Return service name for the given service id.
@@ -150,7 +156,7 @@ class InternalTopology(object):
         return None
 
     def get_services(self):
-        """Return list of service ids previously added to this topology.
+        """Return list of previously added service ids.
         """
         return self._state.get("services", {}).keys()
 
@@ -165,39 +171,72 @@ class InternalTopology(object):
                     service_id, relations))
         del self._state["services"][service_id]
 
-    def add_service_unit(self, service_id, unit_id):
+    def add_service_unit(self, service_id, unit_id, container_id=None):
         """Register unit_id under service_id in this topology state.
 
         The new unit id registered will get a sequence number assigned
         to it.  The sequence number increases monotonically for each
         service, and is helpful to provide nice unit names for users.
 
-        @return: The sequence number assigned to the unit_id.
+        :param container_id: optional unit_id of the principal
+        service unit to which the new unit is subordinate. Defaults to
+        None. If a `container` scoped relationship to the service of
+        the principal node doesn't exist InternalTopologyError is
+        raised.
+
+        :return: The sequence number assigned to the unit_id.
+
         """
         self._assert_service(service_id)
         services = self._state["services"]
         for some_service_id in services:
             if unit_id in services[some_service_id]["units"]:
-                raise InternalTopologyError("Unit %s already in service: %s" %
-                                            (unit_id, some_service_id))
+                raise InternalTopologyError(
+                    "Unit %s already in service: %s" %
+                    (unit_id, some_service_id))
+
+        if container_id is not None:
+            principal_unit_service = self.get_service_unit_service(
+                container_id)
+            relations = self.get_relations_for_service(service_id)
+            found_container_relation = False
+
+            for relation in relations:
+                relation_id = relation["relation_id"]
+                if (self.relation_has_service(
+                        relation_id,
+                        principal_unit_service) and
+                    self.get_relation_scope(relation_id) ==
+                    "container"):
+                    found_container_relation = True
+                    break
+
+            if not found_container_relation:
+                raise InternalTopologyError(
+                    "Attempted to add subordinate unit "
+                    "without container relation")
+
         service = services[service_id]
         services[service_id]["units"][unit_id] = unit = {}
-        unit["sequence"] = self._state["unit-sequence"][service["name"]]
+        unit["sequence"] = self._state[ \
+                                "unit-sequence"][service["name"]]
+        unit["container"] = container_id
         self._state["unit-sequence"][service["name"]] += 1
         return unit["sequence"]
 
     def has_service_unit(self, service_id, unit_id):
-        """Return True if unit_id was previously added under service_id.
+        """Return True if unit_id was exists under service_id.
         """
         self._assert_service(service_id)
         service = self._state["services"][service_id]
         return unit_id in service.get("units", self._nil_dict)
 
     def get_service_units(self, service_id):
-        """Return list of unit_id previously regstered under service_id.
+        """Return list of unit_id registered under service_id.
         """
         self._assert_service(service_id)
-        service = self._state["services"].get(service_id, self._nil_dict)
+        service = self._state["services"].get(service_id,
+                                              self._nil_dict)
         return service.get("units", self._nil_dict).keys()
 
     def get_service_unit_service(self, unit_id):
@@ -206,7 +245,8 @@ class InternalTopology(object):
         for service_id, service in services.iteritems():
             if unit_id in service["units"]:
                 return service_id
-        raise InternalTopologyError("Service unit ID %s not found" % unit_id)
+        raise InternalTopologyError("Service unit ID %s not "
+                                    "found" % unit_id)
 
     def get_service_unit_name(self, service_id, unit_id):
         """Return the user-oriented name for the given unit."""
@@ -227,11 +267,39 @@ class InternalTopology(object):
     def get_service_unit_name_from_id(self, unit_id):
         """Retrieve the user-oriented name from the given unit.
 
-        A simple convience accessor.
+        A simple convenience accessor.
         """
         service_id = self.get_service_unit_service(unit_id)
         return self.get_service_unit_name(service_id, unit_id)
 
+    def get_service_unit_principal(self, unit_id):
+        services = self._state.get("services", self._nil_dict)
+        for service_id, service in services.iteritems():
+            if unit_id in service["units"]:
+                unit_info = service["units"][unit_id]
+                return unit_info.get("container")
+
+        raise InternalTopologyError("Service unit ID %s not "
+                                    "found" % unit_id)
+
+    def get_service_unit_container(self, unit_id):
+        """Return information about the container of a unit.
+
+        If the unit_id has a container this method returns
+        (service_id, service_name, sequence, container_id). Otherwise
+        it returns None.
+        """
+        container_id = self.get_service_unit_principal(unit_id)
+        if container_id is None:
+            return None
+
+        service_id = self.get_service_unit_service(container_id)
+        container_unit_name = self.get_service_unit_name(service_id,
+                                                         container_id)
+        service_name, sequence = container_unit_name.rsplit("/", 1)
+        sequence = int(sequence)
+        return (service_id, service_name, sequence, container_id)
+
     def remove_service_unit(self, service_id, unit_id):
         """Remove unit_id from under service_id in the topology state.
         """
@@ -257,22 +325,25 @@ class InternalTopology(object):
         unit = self._state["services"][service_id]["units"][unit_id]
         return unit["sequence"]
 
-    def assign_service_unit_to_machine(self, service_id, unit_id, machine_id):
+    def assign_service_unit_to_machine(self, service_id, unit_id,
+                                       machine_id):
         """Assign the given unit_id to the provided machine_id.
 
-        The unit_id must exist and be in an unassigned state for this to work.
+        The unit_id must exist and be in an unassigned state for
+        this to work.
         """
         self._assert_service_unit(service_id, unit_id)
         self._assert_machine(machine_id)
         unit = self._state["services"][service_id]["units"][unit_id]
         if "machine" in unit:
             raise InternalTopologyError(
-                "Service unit %s in service %s already assigned to a machine."
+                "Service unit %s in service %s already "
+                "assigned to a machine."
                 % (unit_id, service_id))
         unit["machine"] = machine_id
 
     def get_service_unit_machine(self, service_id, unit_id):
-        """Return the machine_id the given unit_id is assigned to, or None.
+        """Return the machine_id the unit_id is assigned to, or None.
         """
         self._assert_service_unit(service_id, unit_id)
         unit = self._state["services"][service_id]["units"][unit_id]
@@ -290,8 +361,8 @@ class InternalTopology(object):
         unit = self._state["services"][service_id]["units"][unit_id]
         if "machine" not in unit:
             raise InternalTopologyError(
-                "Service unit %s in service %s is not assigned to a machine."
-                % (service_id, unit_id))
+                "Service unit %s in service %s is not "
+                "assigned to a machine." % (service_id, unit_id))
         del unit["machine"]
 
     def get_service_units_in_machine(self, machine_id):
@@ -304,19 +375,23 @@ class InternalTopology(object):
                     units.append(unit_id)
         return units
 
-    def add_relation(self, relation_id, relation_type):
+    def add_relation(self, relation_id, relation_type,
+                     relation_scope="global"):
         """Add a relation with given id and of the given type.
         """
         relations = self._state.setdefault("relations", {})
         if relation_id in relations:
             raise InternalTopologyError(
                 "Relation id %r already in use" % relation_id)
-        relations[relation_id] = (relation_type, {})
+        relations[relation_id] = dict(interface=relation_type,
+                                      scope=relation_scope,
+                                      services=dict())
 
     def has_relation(self, relation_id):
-        """Return True if relation with the given id exists in the topology.
+        """Return True if relation with relation_id exists.
         """
-        return relation_id in self._state.get("relations", self._nil_dict)
+        return relation_id in self._state.get(
+            "relations", self._nil_dict)
 
     def get_relations(self):
         """Returns a list of relation_id in the topology.
@@ -327,20 +402,26 @@ class InternalTopology(object):
         """Get all the services associated to the relation.
         """
         self._assert_relation(relation_id)
-        relation_type, services = self._state["relations"][relation_id]
-        return services
+        relation_data = self._state["relations"][relation_id]
+        return relation_data["services"]
 
     def get_relation_type(self, relation_id):
-        """Get the type of a relation."""
+        """Get the type of a relation (its interface name)."""
         self._assert_relation(relation_id)
-        relation_type, services = self._state["relations"][relation_id]
-        return relation_type
+        relation_data = self._state["relations"][relation_id]
+        return relation_data["interface"]
+
+    def get_relation_scope(self, relation_id):
+        """Get the scope of a relation."""
+        self._assert_relation(relation_id)
+        relation_data = self._state["relations"][relation_id]
+        return relation_data["scope"]
 
     def relation_has_service(self, relation_id, service_id):
         """Return if `service_id` is assigned to `relation_id`."""
         relations = self._state.get("relations", self._nil_dict)
-        relation_type, services = relations.get(relation_id,
-                                                (None, self._nil_dict))
+        relation_data = relations.get(relation_id, self._nil_dict)
+        services = relation_data.get("services", self._nil_dict)
         return service_id in services
 
     def remove_relation(self, relation_id):
@@ -349,7 +430,8 @@ class InternalTopology(object):
         self._assert_relation(relation_id)
         del self._state["relations"][relation_id]
 
-    def assign_service_to_relation(self, relation_id, service_id, name, role):
+    def assign_service_to_relation(self, relation_id, service_id,
+                                   name, role):
         """Associate a service to a relation.
 
         @param role: The relation role of the service.
@@ -357,18 +439,19 @@ class InternalTopology(object):
         """
         self._assert_service(service_id)
         self._assert_relation(relation_id)
-        relation_type, services = self._state["relations"][relation_id]
+        relation_data = self._state["relations"][relation_id]
+        services = relation_data["services"]
         for sid in services:
             if sid == service_id:
                 raise InternalTopologyError(
-                    "Service %r is already assigned to relation %r" % (
-                        service_id, relation_id))
+                    "Service %r is already assigned "
+                    "to relation %r" % (service_id, relation_id))
             service_info = services[sid]
             if service_info["role"] == role:
                 raise InternalTopologyError(
                     ("Another service %r is already providing %r "
-                     "role in relation") % (sid, service_info["role"]))
-
+                     "role in relation") % (sid,
+                                            service_info["role"]))
         services[service_id] = {"role": role, "name": name}
 
     def unassign_service_from_relation(self, relation_id, service_id):
@@ -376,7 +459,8 @@ class InternalTopology(object):
         """
         self._assert_service(service_id)
         self._assert_relation(relation_id)
-        relation_type, services = self._state["relations"][relation_id]
+        relation_data = self._state["relations"][relation_id]
+        services = relation_data["services"]
         if not service_id in services:
             raise InternalTopologyError(
                 "Service %r is not assigned to relation %r" % (
@@ -387,35 +471,49 @@ class InternalTopology(object):
         """Retrieve the service settings for a relation."""
         self._assert_service(service_id)
         self._assert_relation(relation_id)
-        relation_type, services = self._state.get("relations").get(relation_id)
-        if not service_id in services:
+
+        relation_data = self._state.get("relations").get(relation_id)
+        if not service_id in relation_data.get(
+                "services", self._nil_dict):
             raise InternalTopologyError(
                 "Service %r not assigned to relation %r" % (
                     service_id, relation_id))
-        return relation_type, services[service_id]
+        return (relation_data["interface"],
+                relation_data["services"][service_id])
 
     def get_relations_for_service(self, service_id):
-        """Given a service id we should be able to retrieve its relations."""
+        """Given a service id retrieve its relations."""
         self._assert_service(service_id)
         relations = []
-        for relation_id, (relation_type, services) in self._state.get(
-            "relations", self._nil_dict).items():
-            if service_id in services:
-                relations.append((
-                    relation_id, relation_type, services[service_id]))
+
+        relations_dict = self._state.get("relations", self._nil_dict)
+        for relation_id, relation_data in relations_dict.items():
+            services = relation_data.get("services")
+            if services and service_id in services:
+                relations.append(dict(
+                    relation_id=relation_id,
+                    interface=relation_data["interface"],
+                    scope=relation_data["scope"],
+                    service=services[service_id]))
         return relations
 
     def _assert_relation(self, relation_id):
-        if relation_id not in self._state.get("relations", self._nil_dict):
-            raise InternalTopologyError("Relation not found: %s" % relation_id)
+        if relation_id not in self._state.get(
+                "relations", self._nil_dict):
+            raise InternalTopologyError(
+                "Relation not found: %s" % relation_id)
 
     def _assert_machine(self, machine_id):
-        if machine_id not in self._state.get("machines", self._nil_dict):
-            raise InternalTopologyError("Machine not found: %s" % machine_id)
+        if machine_id not in self._state.get(
+                "machines", self._nil_dict):
+            raise InternalTopologyError(
+                "Machine not found: %s" % machine_id)
 
     def _assert_service(self, service_id):
-        if service_id not in self._state.get("services", self._nil_dict):
-            raise InternalTopologyError("Service not found: %s" % service_id)
+        if service_id not in self._state.get(
+                "services", self._nil_dict):
+            raise InternalTopologyError(
+                "Service not found: %s" % service_id)
 
     def _assert_service_unit(self, service_id, unit_id):
         self._assert_service(service_id)
@@ -426,7 +524,7 @@ class InternalTopology(object):
                     unit_id, service_id))
 
     def has_relation_between_endpoints(self, endpoints):
-        """Examine topology to check if relation exists between `endpoints`.
+        """Check if relation exists between `endpoints`.
 
         The relation, with a ``relation type`` common to the
         endpoints, must exist between all endpoints (presumably one
@@ -436,16 +534,21 @@ class InternalTopology(object):
           relations:
             relation-0000000000:
             - mysql
+            - global
             - service-0000000000: {name: db, role: client}
               service-0000000001: {name: server, role: server}
         """
-        service_ids = dict((e, self.find_service_with_name(e.service_name))
-                           for e in endpoints)
+        service_ids = dict((e, self.find_service_with_name(
+            e.service_name)) for e in endpoints)
         relations = self._state.get("relations", self._nil_dict)
-        for _, services in relations.itervalues():
+        for relation_data in relations.itervalues():
+            scope = relation_data["scope"]
+            services = relation_data["services"]
             for endpoint in endpoints:
                 service = services.get(service_ids[endpoint])
-                if not service or service["name"] != endpoint.relation_name:
+                if not service or service["name"] != \
+                   endpoint.relation_name or \
+                   scope != endpoint.relation_scope:
                     break
             else:
                 return True
@@ -453,12 +556,14 @@ class InternalTopology(object):
 
     def get_relation_between_endpoints(self, endpoints):
         """Return relation id existing between `endpoints` or None"""
-        service_ids = dict((e, self.find_service_with_name(e.service_name))
+        service_ids = dict((e, self.find_service_with_name(
+            e.service_name))
                            for e in endpoints)
         relations = self._state.get("relations", self._nil_dict)
-        for relation_id, service_info in relations.iteritems():
-            relation_type, services = service_info
-            if relation_type != endpoints[0].relation_type:
+        for relation_id, relation_data in relations.iteritems():
+            interface = relation_data["interface"]
+            services = relations[relation_id]["services"]
+            if interface != endpoints[0].relation_type:
                 continue
             for endpoint in endpoints:
                 service = services.get(service_ids[endpoint])
