@@ -2,7 +2,6 @@ import os
 import logging
 import shutil
 import tempfile
-import yaml
 
 from twisted.internet.defer import (
     inlineCallbacks, DeferredLock, DeferredList, returnValue)
@@ -10,6 +9,7 @@ from twisted.internet.defer import (
 from juju.errors import CharmUpgradeError
 from juju.hooks.invoker import Invoker
 from juju.hooks.scheduler import HookScheduler
+from juju.lib import serializer
 from juju.state.hook import (
     DepartedRelationHookContext, HookContext, RelationChange)
 from juju.state.errors import StopWatcher, UnitRelationStateNotFound
@@ -493,7 +493,7 @@ class UnitLifecycle(object):
         state_dict = {}
         for relation_wf in self._relations.itervalues():
             state_dict.update(relation_wf.get_relation_info())
-        state = yaml.dump(state_dict)
+        state = serializer.dump(state_dict)
         temp_path = self._known_relations_path + "~"
 
         with open(temp_path, "w") as f:
@@ -519,7 +519,7 @@ class UnitLifecycle(object):
         relations_by_id = dict((r.internal_relation_id, r) for r in relations)
 
         with open(self._known_relations_path) as f:
-            known_relations = yaml.load(f.read())
+            known_relations = serializer.load(f.read())
 
         for relation_id, relation_info in known_relations.items():
             if relation_id in relations_by_id:
