@@ -6,7 +6,6 @@ import shutil
 import stat
 import sys
 
-import yaml
 import zookeeper
 
 from twisted.internet.defer import (inlineCallbacks, Deferred,
@@ -20,6 +19,7 @@ from juju.control.tests.test_upgrade_charm import CharmUpgradeTestBase
 from juju.errors import CharmInvocationError, CharmError, CharmUpgradeError
 from juju.hooks.invoker import Invoker
 from juju.hooks.executor import HookExecutor
+from juju.lib import serializer
 from juju.machine.tests.test_constraints import series_constraints
 
 from juju.state.endpoint import RelationEndpoint
@@ -1083,7 +1083,8 @@ class UnitRelationLifecycleTest(LifecycleTestBase):
         self.assertTrue(self.lifecycle.watching)
         self.assertFalse(self.lifecycle.executing)
 
-        rel_states["unit_relation"].set_data(yaml.dump(dict(hello="world")))
+        rel_states["unit_relation"].set_data(
+            serializer.dump(dict(hello="world")))
         # Sleep to give an error a chance.
         yield self.sleep(0.1)
         self.assertFalse(changed_executed.called)
